# this python script performs the same calculation as example_da.ipynb


import json
import os
import argparse
import datetime

from aivis_engine_v2_da_sdk_python import (
    EngineLogger,
    DependencyAnalysisSetup,
    DependencyAnalysisData,
    DependencyAnalysis,
)

# ---------------------------------------------------------------------------------
# Data location

ANALYSIS_DATA_FILE = "train_da.csv"


# Input and output folder specified via args
def from_args():
    parser = argparse.ArgumentParser(
        description="Example project for dependency analysis"
    )
    # data is expected in input folder
    parser.add_argument(
        "--input", "-i", type=str, nargs=1, required=True, help="Path to input folder"
    )
    # output files will be written to output folder
    parser.add_argument(
        "--output", "-o", type=str, nargs=1, required=True, help="Path to output folder"
    )
    args = parser.parse_args()
    # create output folder if not existing
    if not os.path.exists(args.output[0]):
        os.mkdir(args.output[0])
    return {
        "input_folder": args.input[0],
        "output_folder": args.output[0],
    }


# ---------------------------------------------------------------------------------
# Logger and Threads


class Logger(EngineLogger):
    def log(self, level, thread, module, message):
        print(
            "%s [%s] [%s] %s - %s"
            % (datetime.datetime.now(), level, thread, module, message)
        )


def setup_logger_and_threads():
    print("Initializing DependencyAnalysisSetup: logging and threads ...")
    DependencyAnalysisSetup.register_logger(Logger())
    max_threads = 0  # number of threads will be chosen automatically with 0
    DependencyAnalysisSetup.init_thread_count(max_threads)
    print("\t... done!")
    return


# ---------------------------------------------------------------------------------
# Reading and Writing


def read_json(filename):
    print("\t... reading file '%s' ..." % filename)
    with open(filename, "r") as file:
        return json.loads(file.read())


def write_json(filename, obj):
    print("\t... writing file '%s' ..." % filename)
    with open(filename, "w") as file:
        file.write(json.dumps(obj, indent=2))
    return


# ---------------------------------------------------------------------------------
# MAIN


def main():
    folder_config = from_args()  # parsing args as folder_config
    setup_logger_and_threads()

    print("Loading analysis data ...")
    analysis_data = DependencyAnalysisData.create()

    files_filter = {"includeFiles": [ANALYSIS_DATA_FILE]}
    files_reader_config = json.dumps(
        {
            "folder": folder_config["input_folder"],
            "filesFilter": files_filter,
        }
    )

    analysis_data.read_files(files_reader_config)

    # print some signal to illustrate type DtoFloatDataPoint
    print("Example signal!")
    some_signal_id = "S_6"
    print("Signal id:", some_signal_id)
    some_signal_data_points = analysis_data.get_float_signal(some_signal_id)
    print("Signal length:", len(some_signal_data_points))
    print("Signal entry number 42:", some_signal_data_points[42])
    print(
        "Signal entry number 42 timestamp (unix/millisec):",
        some_signal_data_points[42].timestamp,
    )
    print("Signal entry number 42 value:", some_signal_data_points[42].value)

    # ---- ANALYSIS ----

    print("Creating analysis ...")
    # build analysis config - for more options see documentation
    analysis_config = json.dumps(
        {
            "sampling": {"maximalSampleCount": 10000},
            "clustering": {"minimalClusterSize": 1, "outputMultipleZoomLevels": True},
        }
    )

    # create analysis
    analysis = DependencyAnalysis.create(analysis_data, analysis_config)
    print("\t... done!")

    print("Writing output files...")
    analysis_report = json.loads(analysis.get_report())  # analysis infos
    write_json(
        os.path.join(folder_config["output_folder"], "analysis-report.json"),
        analysis_report,
    )
    print("\t... done!")

    print("Destroy everything ...")
    # destroying can be done once the respective object is not needed anymore - do this if memory is an issue
    analysis_data.destroy()
    analysis.destroy()

    print("\t... done!")

    print("Generating report ...")

    # ---- Toolbox HTML Report ----
    from aivis_engine_v2_toolbox.api import build_dependency_analysis_report

    build_dependency_analysis_report(
        {
            "title": "Aivis Engine V2 - Dependency Analysis Example: Damadics",
            "analysisReportFile": os.path.join(
                folder_config["output_folder"], "analysis-report.json"
            ),
            "outputFile": os.path.join(
                folder_config["output_folder"],
                "aivis-engine-v2-dependency-analysis-example.html",
            ),
        }
    )

    print("ALL DONE!")

    return


if __name__ == "__main__":
    main()
